@@ -13,7 +13,10 @@
 """ Unit test for runner module.
     """
 
-import __builtin__ as builtins
+try:
+    import __builtin__ as builtins
+except ImportError:
+    import builtins
 import os
 import sys
 import tempfile
@@ -26,16 +29,16 @@ from test_pidlockfile import (
     setup_pidfile_fixtures,
     make_pidlockfile_scenarios,
     setup_lockfile_method_mocks,
-    )
+)
 from test_daemon import (
     setup_streams_fixtures,
-    )
+)
 import daemon.daemon
 
 from daemon import pidlockfile
 from daemon import runner
 
-
+
 class Exception_TestCase(scaffold.Exception_TestCase):
     """ Test cases for module exception classes. """
 
@@ -62,7 +65,7 @@ class Exception_TestCase(scaffold.Exception_TestCase):
                 ),
             }
 
-
+
 def make_runner_scenarios():
     """ Make a collection of scenarios for testing DaemonRunner instances. """
 
@@ -77,7 +80,7 @@ def make_runner_scenarios():
             },
         }
 
-    for scenario in scenarios.values():
+    for scenario in list(scenarios.values()):
         if 'pidlockfile_scenario_name' in scenario:
             pidlockfile_scenario = pidlockfile_scenarios.pop(
                 scenario['pidlockfile_scenario_name'])
@@ -88,7 +91,7 @@ def make_runner_scenarios():
 
     return scenarios
 
-
+
 def set_runner_scenario(testcase, scenario_name, clear_tracker=True):
     """ Set the DaemonRunner test scenario for the test case. """
     scenarios = testcase.runner_scenarios
@@ -119,13 +122,13 @@ def setup_runner_fixtures(testcase):
 
     testcase.mock_stderr = FakeFileDescriptorStringIO()
     scaffold.mock(
-        u"sys.stderr",
+        "sys.stderr",
         mock_obj=testcase.mock_stderr,
         tracker=testcase.mock_tracker)
 
     simple_scenario = testcase.runner_scenarios['simple']
 
-    testcase.lockfile_class_name = u"pidlockfile.TimeoutPIDLockFile"
+    testcase.lockfile_class_name = "pidlockfile.TimeoutPIDLockFile"
 
     testcase.mock_runner_lock = scaffold.Mock(
         testcase.lockfile_class_name,
@@ -147,23 +150,23 @@ def setup_runner_fixtures(testcase):
             self.pidfile_timeout = simple_scenario['pidfile_timeout']
 
         run = scaffold.Mock(
-            u"TestApp.run",
+            "TestApp.run",
             tracker=testcase.mock_tracker)
 
     testcase.TestApp = TestApp
 
     scaffold.mock(
-        u"daemon.runner.DaemonContext",
+        "daemon.runner.DaemonContext",
         returns=scaffold.Mock(
-            u"DaemonContext",
+            "DaemonContext",
             tracker=testcase.mock_tracker),
         tracker=testcase.mock_tracker)
 
     testcase.test_app = testcase.TestApp()
 
-    testcase.test_program_name = u"bazprog"
+    testcase.test_program_name = "bazprog"
     testcase.test_program_path = (
-        u"/foo/bar/%(test_program_name)s" % vars(testcase))
+        "/foo/bar/%(test_program_name)s" % vars(testcase))
     testcase.valid_argv_params = {
         'start': [testcase.test_program_path, 'start'],
         'stop': [testcase.test_program_path, 'stop'],
@@ -180,16 +183,16 @@ def setup_runner_fixtures(testcase):
         return result
 
     scaffold.mock(
-        u"builtins.open",
+        "builtins.open",
         returns_func=mock_open,
         tracker=testcase.mock_tracker)
 
     scaffold.mock(
-        u"os.kill",
+        "os.kill",
         tracker=testcase.mock_tracker)
 
     scaffold.mock(
-        u"sys.argv",
+        "sys.argv",
         mock_obj=testcase.valid_argv_params['start'],
         tracker=testcase.mock_tracker)
 
@@ -197,7 +200,7 @@ def setup_runner_fixtures(testcase):
 
     testcase.scenario = NotImplemented
 
-
+
 class DaemonRunner_TestCase(scaffold.TestCase):
     """ Test cases for DaemonRunner class. """
 
@@ -207,7 +210,7 @@ class DaemonRunner_TestCase(scaffold.TestCase):
         set_runner_scenario(self, 'simple')
 
         scaffold.mock(
-            u"runner.DaemonRunner.parse_args",
+            "runner.DaemonRunner.parse_args",
             tracker=self.mock_tracker)
 
         self.test_instance = runner.DaemonRunner(self.test_app)
@@ -222,7 +225,7 @@ class DaemonRunner_TestCase(scaffold.TestCase):
 
     def test_parses_commandline_args(self):
         """ Should parse commandline arguments. """
-        expect_mock_output = u"""\
+        expect_mock_output = """\
             Called runner.DaemonRunner.parse_args()
             ...
             """
@@ -245,16 +248,16 @@ class DaemonRunner_TestCase(scaffold.TestCase):
         pidfile_path = object()
         self.test_app.pidfile_path = pidfile_path
         expect_error = ValueError
-        self.failUnlessRaises(
+        self.assertRaises(
             expect_error,
             runner.DaemonRunner, self.test_app)
 
     def test_error_when_pidfile_path_not_absolute(self):
         """ Should raise ValueError when PID file path not absolute. """
-        pidfile_path = u"foo/bar.pid"
+        pidfile_path = "foo/bar.pid"
         self.test_app.pidfile_path = pidfile_path
         expect_error = ValueError
-        self.failUnlessRaises(
+        self.assertRaises(
             expect_error,
             runner.DaemonRunner, self.test_app)
 
@@ -263,7 +266,7 @@ class DaemonRunner_TestCase(scaffold.TestCase):
         pidfile_path = self.scenario['pidfile_path']
         pidfile_timeout = self.scenario['pidfile_timeout']
         lockfile_class_name = self.lockfile_class_name
-        expect_mock_output = u"""\
+        expect_mock_output = """\
             ...
             Called %(lockfile_class_name)s(
                 %(pidfile_path)r,
@@ -291,7 +294,7 @@ class DaemonRunner_TestCase(scaffold.TestCase):
         test_app = self.test_app
         expect_file = self.stream_files_by_name['stdin']
         daemon_context = self.test_instance.daemon_context
-        self.failUnlessEqual(expect_file, daemon_context.stdin)
+        self.assertEqual(expect_file, daemon_context.stdin)
 
     def test_daemon_context_has_stdin_in_read_mode(self):
         """ DaemonContext component should open stdin file for read. """
@@ -304,7 +307,7 @@ class DaemonRunner_TestCase(scaffold.TestCase):
         test_app = self.test_app
         expect_file = self.stream_files_by_name['stdout']
         daemon_context = self.test_instance.daemon_context
-        self.failUnlessEqual(expect_file, daemon_context.stdout)
+        self.assertEqual(expect_file, daemon_context.stdout)
 
     def test_daemon_context_has_stdout_in_append_mode(self):
         """ DaemonContext component should open stdout file for append. """
@@ -317,7 +320,7 @@ class DaemonRunner_TestCase(scaffold.TestCase):
         test_app = self.test_app
         expect_file = self.stream_files_by_name['stderr']
         daemon_context = self.test_instance.daemon_context
-        self.failUnlessEqual(expect_file, daemon_context.stderr)
+        self.assertEqual(expect_file, daemon_context.stderr)
 
     def test_daemon_context_has_stderr_in_append_mode(self):
         """ DaemonContext component should open stderr file for append. """
@@ -329,10 +332,10 @@ class DaemonRunner_TestCase(scaffold.TestCase):
         """ DaemonContext component should open stderr file unbuffered. """
         expect_buffering = 0
         daemon_context = self.test_instance.daemon_context
-        self.failUnlessEqual(
+        self.assertEqual(
             expect_buffering, daemon_context.stderr.buffering)
 
-
+
 class DaemonRunner_usage_exit_TestCase(scaffold.TestCase):
     """ Test cases for DaemonRunner.usage_exit method. """
 
@@ -349,7 +352,7 @@ class DaemonRunner_usage_exit_TestCase(scaffold.TestCase):
         """ Should raise SystemExit exception. """
         instance = self.test_instance
         argv = [self.test_program_path]
-        self.failUnlessRaises(
+        self.assertRaises(
             SystemExit,
             instance._usage_exit, argv)
 
@@ -358,16 +361,16 @@ class DaemonRunner_usage_exit_TestCase(scaffold.TestCase):
         instance = self.test_instance
         progname = self.test_program_name
         argv = [self.test_program_path]
-        expect_stderr_output = u"""\
+        expect_stderr_output = """\
             usage: %(progname)s ...
             """ % vars()
-        self.failUnlessRaises(
+        self.assertRaises(
             SystemExit,
             instance._usage_exit, argv)
         self.failUnlessOutputCheckerMatch(
             expect_stderr_output, self.mock_stderr.getvalue())
 
-
+
 class DaemonRunner_parse_args_TestCase(scaffold.TestCase):
     """ Test cases for DaemonRunner.parse_args method. """
 
@@ -377,7 +380,7 @@ class DaemonRunner_parse_args_TestCase(scaffold.TestCase):
         set_runner_scenario(self, 'simple')
 
         scaffold.mock(
-            u"daemon.runner.DaemonRunner._usage_exit",
+            "daemon.runner.DaemonRunner._usage_exit",
             raises=NotImplementedError,
             tracker=self.mock_tracker)
 
@@ -389,7 +392,7 @@ class DaemonRunner_parse_args_TestCase(scaffold.TestCase):
         """ Should emit a usage message and exit if too few arguments. """
         instance = self.test_instance
         argv = [self.test_program_path]
-        expect_mock_output = u"""\
+        expect_mock_output = """\
             Called daemon.runner.DaemonRunner._usage_exit(%(argv)r)
             """ % vars()
         try:
@@ -403,7 +406,7 @@ class DaemonRunner_parse_args_TestCase(scaffold.TestCase):
         instance = self.test_instance
         progname = self.test_program_name
         argv = [self.test_program_path, 'bogus']
-        expect_mock_output = u"""\
+        expect_mock_output = """\
             Called daemon.runner.DaemonRunner._usage_exit(%(argv)r)
             """ % vars()
         try:
@@ -415,24 +418,24 @@ class DaemonRunner_parse_args_TestCase(scaffold.TestCase):
     def test_should_parse_system_argv_by_default(self):
         """ Should parse sys.argv by default. """
         instance = self.test_instance
-        expect_action = u'start'
+        expect_action = 'start'
         argv = self.valid_argv_params['start']
         scaffold.mock(
-            u"sys.argv",
+            "sys.argv",
             mock_obj=argv,
             tracker=self.mock_tracker)
         instance.parse_args()
-        self.failUnlessEqual(expect_action, instance.action)
+        self.assertEqual(expect_action, instance.action)
 
     def test_sets_action_from_first_argument(self):
         """ Should set action from first commandline argument. """
         instance = self.test_instance
-        for name, argv in self.valid_argv_params.items():
+        for name, argv in list(self.valid_argv_params.items()):
             expect_action = name
             instance.parse_args(argv)
-            self.failUnlessEqual(expect_action, instance.action)
+            self.assertEqual(expect_action, instance.action)
+
 
-
 class DaemonRunner_do_action_TestCase(scaffold.TestCase):
     """ Test cases for DaemonRunner.do_action method. """
 
@@ -448,9 +451,9 @@ class DaemonRunner_do_action_TestCase(scaffold.TestCase):
     def test_raises_error_if_unknown_action(self):
         """ Should emit a usage message and exit if action is unknown. """
         instance = self.test_instance
-        instance.action = u'bogus'
+        instance.action = 'bogus'
         expect_error = runner.DaemonRunnerInvalidActionError
-        self.failUnlessRaises(
+        self.assertRaises(
             expect_error,
             instance.do_action)
 
@@ -463,7 +466,7 @@ class DaemonRunner_do_action_start_TestCase(scaffold.TestCase):
         setup_runner_fixtures(self)
         set_runner_scenario(self, 'simple')
 
-        self.test_instance.action = u'start'
+        self.test_instance.action = 'start'
 
     def tearDown(self):
         """ Tear down test fixtures. """
@@ -478,14 +481,19 @@ class DaemonRunner_do_action_start_TestCase(scaffold.TestCase):
         pidfile_path = self.scenario['pidfile_path']
         expect_error = runner.DaemonRunnerStartFailureError
         expect_message_content = pidfile_path
+        exc = None
         try:
             instance.do_action()
-        except expect_error, exc:
-            pass
+        except expect_error as exc_:
+            exc = exc_
         else:
             raise self.failureException(
-                u"Failed to raise " + expect_error.__name__)
-        self.failUnlessIn(unicode(exc.message), expect_message_content)
+                "Failed to raise " + expect_error.__name__)
+        try:
+            message = exc.message
+        except AttributeError:
+            message = exc.args[0]
+        self.failUnlessIn(str(message), expect_message_content)
 
     def test_breaks_lock_if_no_such_process(self):
         """ Should request breaking lock if PID file process is not running. """
@@ -496,10 +504,10 @@ class DaemonRunner_do_action_start_TestCase(scaffold.TestCase):
         pidfile_path = self.scenario['pidfile_path']
         test_pid = self.scenario['pidlockfile_scenario']['pidfile_pid']
         expect_signal = signal.SIG_DFL
-        error = OSError(errno.ESRCH, u"Not running")
+        error = OSError(errno.ESRCH, "Not running")
         os.kill.mock_raises = error
         lockfile_class_name = self.lockfile_class_name
-        expect_mock_output = u"""\
+        expect_mock_output = """\
             ...
             Called os.kill(%(test_pid)r, %(expect_signal)r)
             Called %(lockfile_class_name)s.break_lock()
@@ -512,7 +520,7 @@ class DaemonRunner_do_action_start_TestCase(scaffold.TestCase):
     def test_requests_daemon_context_open(self):
         """ Should request the daemon context to open. """
         instance = self.test_instance
-        expect_mock_output = u"""\
+        expect_mock_output = """\
             ...
             Called DaemonContext.open()
             ...
@@ -524,7 +532,7 @@ class DaemonRunner_do_action_start_TestCase(scaffold.TestCase):
         """ Should emit start message to stderr. """
         instance = self.test_instance
         current_pid = self.scenario['pid']
-        expect_stderr = u"""\
+        expect_stderr = """\
             started with pid %(current_pid)d
             """ % vars()
         instance.do_action()
@@ -534,7 +542,7 @@ class DaemonRunner_do_action_start_TestCase(scaffold.TestCase):
     def test_requests_app_run(self):
         """ Should request the application to run. """
         instance = self.test_instance
-        expect_mock_output = u"""\
+        expect_mock_output = """\
             ...
             Called TestApp.run()
             """
@@ -550,7 +558,7 @@ class DaemonRunner_do_action_stop_TestCase(scaffold.TestCase):
         setup_runner_fixtures(self)
         set_runner_scenario(self, 'pidfile-locked')
 
-        self.test_instance.action = u'stop'
+        self.test_instance.action = 'stop'
 
         self.mock_runner_lock.is_locked.mock_returns = True
         self.mock_runner_lock.i_am_locking.mock_returns = False
@@ -572,13 +580,14 @@ class DaemonRunner_do_action_stop_TestCase(scaffold.TestCase):
         pidfile_path = self.scenario['pidfile_path']
         expect_error = runner.DaemonRunnerStopFailureError
         expect_message_content = pidfile_path
+        exc = None
         try:
             instance.do_action()
-        except expect_error, exc:
-            pass
+        except expect_error as exc_:
+            exc = exc_
         else:
             raise self.failureException(
-                u"Failed to raise " + expect_error.__name__)
+                "Failed to raise " + expect_error.__name__)
         scaffold.mock_restore()
         self.failUnlessIn(str(exc), expect_message_content)
 
@@ -588,10 +597,10 @@ class DaemonRunner_do_action_stop_TestCase(scaffold.TestCase):
         pidfile_path = self.scenario['pidfile_path']
         test_pid = self.scenario['pidlockfile_scenario']['pidfile_pid']
         expect_signal = signal.SIG_DFL
-        error = OSError(errno.ESRCH, u"Not running")
+        error = OSError(errno.ESRCH, "Not running")
         os.kill.mock_raises = error
         lockfile_class_name = self.lockfile_class_name
-        expect_mock_output = u"""\
+        expect_mock_output = """\
             ...
             Called %(lockfile_class_name)s.break_lock()
             """ % vars()
@@ -604,7 +613,7 @@ class DaemonRunner_do_action_stop_TestCase(scaffold.TestCase):
         instance = self.test_instance
         test_pid = self.scenario['pidlockfile_scenario']['pidfile_pid']
         expect_signal = signal.SIGTERM
-        expect_mock_output = u"""\
+        expect_mock_output = """\
             ...
             Called os.kill(%(test_pid)r, %(expect_signal)r)
             """ % vars()
@@ -617,19 +626,20 @@ class DaemonRunner_do_action_stop_TestCase(scaffold.TestCase):
         instance = self.test_instance
         test_pid = self.scenario['pidlockfile_scenario']['pidfile_pid']
         pidfile_path = self.scenario['pidfile_path']
-        error = OSError(errno.EPERM, u"Nice try")
+        error = OSError(errno.EPERM, "Nice try")
         os.kill.mock_raises = error
         expect_error = runner.DaemonRunnerStopFailureError
         expect_message_content = str(test_pid)
+        exc = None
         try:
             instance.do_action()
-        except expect_error, exc:
-            pass
+        except expect_error as exc_:
+            exc = exc_
         else:
             raise self.failureException(
-                u"Failed to raise " + expect_error.__name__)
+                "Failed to raise " + expect_error.__name__)
         scaffold.mock_restore()
-        self.failUnlessIn(unicode(exc), expect_message_content)
+        self.failUnlessIn(str(exc), expect_message_content)
 
 
 class DaemonRunner_do_action_restart_TestCase(scaffold.TestCase):
@@ -640,7 +650,7 @@ class DaemonRunner_do_action_restart_TestCase(scaffold.TestCase):
         setup_runner_fixtures(self)
         set_runner_scenario(self, 'pidfile-locked')
 
-        self.test_instance.action = u'restart'
+        self.test_instance.action = 'restart'
 
     def tearDown(self):
         """ Tear down test fixtures. """
@@ -650,12 +660,12 @@ class DaemonRunner_do_action_restart_TestCase(scaffold.TestCase):
         """ Should request stop, then start. """
         instance = self.test_instance
         scaffold.mock(
-            u"daemon.runner.DaemonRunner._start",
+            "daemon.runner.DaemonRunner._start",
             tracker=self.mock_tracker)
         scaffold.mock(
-            u"daemon.runner.DaemonRunner._stop",
+            "daemon.runner.DaemonRunner._stop",
             tracker=self.mock_tracker)
-        expect_mock_output = u"""\
+        expect_mock_output = """\
             Called daemon.runner.DaemonRunner._stop()
             Called daemon.runner.DaemonRunner._start()
             """
